import os

from .consts import (
    _DEFAULT_BASE_URL,
    QUALIFIRE_API_KEY_ENV_VAR,
    QUALIFIRE_BASE_URL_ENV_VAR,
)


def get_api_key() -> str:
    api_key = os.getenv(QUALIFIRE_API_KEY_ENV_VAR)
    if not api_key:
        raise ValueError("Qualifire API Key not found")
    return api_key


def get_base_url() -> str:
    base_url = os.getenv(QUALIFIRE_BASE_URL_ENV_VAR)
    if not base_url:
        return _DEFAULT_BASE_URL
    return base_url



from typing import Any, Dict, List, Optional

from dataclasses import dataclass, field
from enum import Enum


class ModelMode(str, Enum):
    SPEED = "speed"
    BALANCED = "balanced"
    QUALITY = "quality"


class PolicyTarget(str, Enum):
    INPUT = "input"
    OUTPUT = "output"
    BOTH = "both"


@dataclass
class LLMToolDefinition:
    name: str
    description: str
    parameters: Dict[str, Any]


@dataclass
class LLMToolCall:
    name: str
    arguments: Dict[str, Any]
    id: Optional[str]


@dataclass
class LLMMessage:
    role: str
    content: str
    tool_calls: Optional[List[LLMToolCall]] = None


@dataclass
class SyntaxCheckArgs:
    args: str


@dataclass
class EvaluationRequest:
    input: Optional[str] = None
    output: Optional[str] = None
    messages: Optional[List[LLMMessage]] = field(default_factory=list)
    available_tools: Optional[List[LLMToolDefinition]] = None
    dangerous_content_check: bool = False  # Deprecated: use content_moderation_check
    hallucinations_check: bool = False
    harassment_check: bool = False  # Deprecated: use content_moderation_check
    hate_speech_check: bool = False  # Deprecated: use content_moderation_check
    pii_check: bool = False
    prompt_injections: bool = False
    sexual_content_check: bool = False  # Deprecated: use content_moderation_check
    grounding_check: bool = False
    syntax_checks: Optional[Dict[str, SyntaxCheckArgs]] = None
    assertions: Optional[List[str]] = field(default_factory=list)
    tool_selection_quality_check: bool = False  # Deprecated: use tool_use_quality_check
    tool_use_quality_check: bool = False
    content_moderation_check: bool = False
    tsq_mode: Optional[ModelMode] = None  # Deprecated: use tuq_mode
    tuq_mode: Optional[ModelMode] = None
    consistency_mode: ModelMode = ModelMode.BALANCED
    assertions_mode: ModelMode = ModelMode.BALANCED
    grounding_mode: ModelMode = ModelMode.BALANCED
    hallucinations_mode: ModelMode = ModelMode.BALANCED
    grounding_multi_turn_mode: bool = False
    policy_multi_turn_mode: bool = False
    policy_target: PolicyTarget = PolicyTarget.BOTH

    def __post_init__(self):
        self._validate_messages_input_output()
        self._validate_tsq_requirements()
        self._handle_deprecated_content_checks()

    def _validate_messages_input_output(self):
        if not self.messages and not self.input and not self.output:
            raise ValueError(
                "At least one of messages, input, or output must be set",
            )

    def _validate_tsq_requirements(self):
        if (
            self.tool_selection_quality_check or self.tool_use_quality_check
        ) and not self.messages:
            raise ValueError(
                "messages must be provided in conjunction "
                "with tool_use_quality_check=True."
            )
        if (
            self.tool_selection_quality_check or self.tool_use_quality_check
        ) and not self.available_tools:
            raise ValueError(
                "available_tools must be provided in conjunction "
                "with tool_use_quality_check=True."
            )

    def _handle_deprecated_content_checks(self):
        """Auto-set content_moderation_check if deprecated fields are set."""
        if (
            self.dangerous_content_check
            or self.harassment_check
            or self.hate_speech_check
            or self.sexual_content_check
        ):
            self.content_moderation_check = True


@dataclass
class EvaluationInvokeRequest:
    evaluation_id: str
    input: Optional[str] = None
    output: Optional[str] = None
    messages: Optional[List[LLMMessage]] = None
    available_tools: Optional[List[LLMToolDefinition]] = None

    def __post_init__(self):
        self._validate_messages_input_output()

    def _validate_messages_input_output(self):
        if not self.messages and not self.input and not self.output:
            raise ValueError(
                "At least one of messages, input, or output must be set",
            )


@dataclass
class EvaluationResult:
    claim: str
    confidence_score: int
    label: str
    name: str
    quote: str
    reason: str
    score: int
    flagged: bool


@dataclass
class EvaluationResultItem:
    results: List[EvaluationResult]
    type: str


@dataclass
class EvaluationResponse:
    evaluationResults: List[EvaluationResultItem]
    score: int
    status: str



from typing import Any, Callable, Optional, TypeVar

import os
import sys

try:
    from traceloop.sdk import Traceloop

    traceloop_installed = True
except ImportError:
    traceloop_installed = False

from .utils import get_api_key, get_base_url

R = TypeVar("R")


def __suppress_prints(
    func: Callable[..., R],
    *args: Any,
    **kwargs: Any,
) -> R:
    original_stdout = sys.stdout
    sys.stdout = open(os.devnull, "w")
    try:
        return func(*args, **kwargs)
    finally:
        sys.stdout.close()
        sys.stdout = original_stdout


def __configure_tracer(api_key: str) -> None:
    # traceloop uses env for those configs
    os.environ["TRACELOOP_METRICS_ENABLED"] = "false"
    os.environ["TRACELOOP_LOGGING_ENABLED"] = "false"

    __suppress_prints(
        Traceloop.init,
        app_name="qualifire-agent",
        api_endpoint=f"{get_base_url()}/api/telemetry",  # /v1/traces is automatically added  # noqa: E501
        headers={"X-Qualifire-API-Key": api_key},
        telemetry_enabled=False,
        traceloop_sync_enabled=False,
    )


def init(api_key: Optional[str] = None) -> None:
    if not traceloop_installed:
        if sys.version_info < (3, 10):
            raise RuntimeError("qualifire.init requires Python 3.10 or higher")
        else:
            raise RuntimeError("Dependency error, please reinstall qualifire-sdk")

    api_key = api_key or get_api_key()
    __configure_tracer(api_key)



"""Qualifire Python SDK"""

import logging

from . import client, consts, tracer_init, types, utils
from .tracer_init import init

logger = logging.getLogger("qualifire")


from importlib import metadata as importlib_metadata


def get_version() -> str:
    try:
        return importlib_metadata.version(__name__)
    except importlib_metadata.PackageNotFoundError:  # pragma: no cover
        return "unknown"


version: str = get_version()

__all__ = [
    "client",
    "types",
    "init",
    "version",
]
